#!/bin/bash
# Build documentation and upload to bitbucket pages
# Runs as a part of the automated testing on CircleCI
set -euxo pipefail

# Script setup
# Notice that it is the "ocellarisbot" user that is used for updating the web page
REPO=ocellarisbot@bitbucket.org:ocellarisproject/ocellarisproject.bitbucket.io.git
CMESSAGE="CircleCI Ocellaris doc update by Ocellarisbot"
cd documentation/


# Install and run Sphinx
pip3 install sphinx --force --user
pip3 install sphinxcontrib-newsfeed --user
pip3 install sphinxcontrib-bibtex --user
pip3 install sphinx-sitemap --user
#pip3 install git+https://github.com/sphinx-contrib/youtube.git --user
pip3 install git+https://github.com/TormodLandet/youtube.git@tormod/allowfullscreen --user
export PATH=~/.local/bin:$PATH
make html

# Only update the web page when on the master branch
if [[ ! "$(git rev-parse --abbrev-ref HEAD)" == "master" ]]; then
  echo "Not on master branch, skipping documentation upload"
  exit 0
fi

# Configure git
# Git commit access given to CircleCI following this procedure
# https://discuss.circleci.com/t/checkout-fails-with-user-key-bitbucket/8735/7
git config --global user.email "nobody@example.com"
git config --global user.name "CircleCI Ocellaris Doc Builder"
git config --global push.default simple


# Clone and update the webpage git repo
git clone --depth 1 $REPO webpage
rm -r webpage/ocellaris
mv _build/html webpage/ocellaris


# Add doc changes to repo
cd webpage/
git add -A


# Exit if there are no changes
if git diff-index --quiet HEAD --; then
  exit 0
fi


# Amend previous commit if it was a CI commit, else
# make a new commit (to avoid filling up the repo
# with autogenerated commits
if [[ "$(git log -1 --pretty=%B)" == *"$CMESSAGE"* ]]; then
  git commit --amend -m "$CMESSAGE"
  git push -f
else
  git commit -m "$CMESSAGE"
  git push
fi
