# Copyright (C) 2015-2019 Tormod Landet
# SPDX-License-Identifier: Apache-2.0

# -*- coding: utf-8 -*-
#
# Ocellaris documentation build configuration file, created by
# sphinx-quickstart on Fri Mar  6 09:57:44 2015.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import sys
import os


# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
main_dir = os.path.abspath('..')
sys.path.insert(0, main_dir)


# No dolfin etc available on ReadTheDocs, see
# http://docs.readthedocs.io/en/latest/faq.html
if os.environ.get('READTHEDOCS') == 'True':
    from unittest.mock import MagicMock

    class Mock(MagicMock):
        @classmethod
        def __getattr__(cls, name):
            return MagicMock()

    MOCK_MODULES = [
        'dolfin',
        'ufl',
        'ufl.classes',
        'numpy',
        'yaml' 'matplotlib',
        'matplotlib.collections',
        'matplotlib.tri',
        'petsc4py',
        'h5py',
    ]
    sys.modules.update((mod_name, Mock()) for mod_name in MOCK_MODULES)


# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.mathjax',
    'sphinx.ext.napoleon',
    'sphinxcontrib.newsfeed',
    'sphinxcontrib.youtube',
    'sphinxcontrib.bibtex',
    'sphinx_sitemap',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'Ocellaris'
copyright = u'2015-2019, Tormod Landet'
html_baseurl = 'https://www.ocellaris.org/'
default_description = ' '.join(
    """
    The Ocellaris user guide, documentation and news blog shows how to use the
    Ocellaris higher order DG FEM Navier-Stokes solver to simulate free surface
    flows such as the air/water flow in breaking ocean waves""".split()
).strip()

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.

# Get the Ocellaris version
init_path = os.path.join(main_dir, 'ocellaris', '__init__.py')
with open(init_path) as f:
    for line in f:
        if line.startswith('__version__'):
            ocellaris_version = line.split('=')[1].strip()[1:-1]

# The short X.Y version.
version = ocellaris_version.split('.dev')[0]  # remove ".devX" postfix
# The full version, including alpha/beta/rc tags.
release = ocellaris_version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
# language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
# today = ''
# Else, today_fmt is used as the format for a strftime call.
# today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build']

# The reST default role (used for this markup: `text`) to use for all
# documents.
# default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
# add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
# modindex_common_prefix = []

# If true, keep warnings as "system message" paragraphs in the built documents.
# keep_warnings = False


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'haiku'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
# html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
# html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
# html_title = None

# A shorter title for the navigation bar.  Default is the same as html_title.
# html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
html_logo = '_static/icon_128.png'

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
# html_favicon = None  # we handle this explicitly in the theme

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']


def setup(app):
    app.add_stylesheet('ocellaris.css')


# Add any extra paths that contain custom files (such as robots.txt or
# .htaccess) here, relative to this directory. These files are copied
# directly to the root of the documentation.
# html_extra_path = []

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
# html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
# html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
# html_additional_pages = {}

# If false, no module index is generated.
# html_domain_indices = True

# If false, no index is generated.
# html_use_index = True

# If true, the index is split into individual pages for each letter.
# html_split_index = False

# If true, links to the reST sources are added to the pages.
# html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
# html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
# html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
# html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = 'Ocellarisdoc'


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    # 'papersize': 'letterpaper',
    # The font size ('10pt', '11pt' or '12pt').
    # 'pointsize': '10pt',
    # Additional stuff for the LaTeX preamble.
    # 'preamble': '',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    ('index', 'Ocellaris.tex', u'Ocellaris Documentation', u'Tormod Landet', 'manual')
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
# latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# If true, show page references after internal links.
# latex_show_pagerefs = False

# If true, show URL addresses after external links.
# latex_show_urls = False

# Documents to append as an appendix to all manuals.
# latex_appendices = []

# If false, no module index is generated.
# latex_domain_indices = True


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [('index', 'ocellaris', u'Ocellaris Documentation', [u'Tormod Landet'], 1)]

# If true, show URL addresses after external links.
# man_show_urls = False


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (
        'index',
        'Ocellaris',
        u'Ocellaris Documentation',
        u'Tormod Landet',
        'Ocellaris',
        'One line description of project.',
        'Miscellaneous',
    )
]

# Documents to append as an appendix to all manuals.
# texinfo_appendices = []

# If false, no module index is generated.
# texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
# texinfo_show_urls = 'footnote'

# If true, do not generate a @detailmenu in the "Top" node's menu.
# texinfo_no_detailmenu = False

###########################################################################

autoclass_content = 'both'
autodoc_default_flags = ['members', 'undoc-members', 'show-inheritance']

###########################################################################
# Monkey-patch functools.wraps
# See: https://github.com/sphinx-doc/sphinx/issues/1711
import functools


def no_op_wraps(func):
    """
    Replaces functools.wraps in order to undo wrapping when generating Sphinx documentation
    """
    if func.__module__ is None or 'ocellaris' not in func.__module__:
        return functools.orig_wraps(func)

    def wrapper(decorator):
        return func

    return wrapper


functools.orig_wraps = functools.wraps
functools.wraps = no_op_wraps

###########################################################################
# Cludge to work around the fact that docutils gives meta information
# only as pre-rendered HTML. Arghh ... :-(


def get_meta_tag_content(name, html_string):
    """
    Given input arguments
        name='myname',
        html_string='<... name="myname" ... content="a" ...>',
    Return the string 'a'.
    If the requested name is not found, return ''
    """
    if not html_string or '<' not in html_string:
        return ''
    for tag in html_string.split('<')[1:]:
        # Check that we can expect to be able to parse this tag
        if 'name="' not in tag:
            print('ERROR: metatag without name found: %r' % tag)
            print('ERROR: html_string=%r' % html_string)
            continue
        if 'content="' not in tag:
            print('ERROR: metatag without content found: %r' % tag)
            print('ERROR: html_string=%r' % html_string)
            continue

        # Parse the generated HTML and check for the correct tag name
        tag_name = tag.split('name="')[1].split('"')[0]
        tag_content = tag.split('content="')[1].split('"')[0]
        if tag_name == name:
            return tag_content
    return ''


html_context = {
    'default_description': default_description,
    'get_meta_tag_content': get_meta_tag_content,
}
